library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
library(Kendall)
library(miceadds)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "aemet-5km_mon/OUTPUTS/"

############Regions##########################
datos <- readxl::read_xlsx("SPSS.xlsx",sheet=3)
datos <- datos[3:5234,11:14]
colnames(datos) <- c("Point","Longitude","Latitude","Region")
datos$Latitude <- round(as.numeric(datos$Latitude),digits=3)
datos$Longitude <- round(as.numeric(datos$Longitude),digits=3)

datos$Region[which(datos$Region=="A")] <- 1
datos$Region[which(datos$Region=="B")] <- 2
datos$Region[which(datos$Region=="C")] <- 3
datos$Region <- as.numeric(datos$Region)

library(ggplot2)
datos$Region <- as.factor(datos$Region)

# png(filename = paste("~/Downloads/RegionesMA.png",sep=""),res=600,width=2000,height=2000)
# 
# print(ggplot(data=datos,aes(x=Longitude,y=Latitude,fill=Region)) + geom_tile() + borders(colour = "grey22",size=0.3)+
#         theme_bw() + coord_quickmap(xlim=c(min(datos$Longitude),max(datos$Longitude)),ylim=c(min(datos$Latitude),max(datos$Latitude)),expand = T) 
# )
# dev.off()

#Genero retícula
lat <- seq(36.075,43.675,by=0.1)
lon <- seq(-8.025,3.275,by=0.1)

cp <- loadGridData(dataset = "eobs_tasmax_spain.nc",years=2000,var="tasmax")

cp <- interpGrid(cp,new.coordinates = list(x=lon,y=lat))
cp <- redim(climatology(cp),drop=T)
cp <- redim(gridArithmetics(cp,0,operator = "*"),drop=T)

for(i in 1:dim(cp$Data)[1]){
  for(j in 1:dim(cp$Data)[2]){
    #    cp$Data[i,j] <- NaN
    if(sum(which(datos$Latitude==lat[i] & datos$Longitude==lon[j]))!=0)
      cp$Data[i,j] <- datos$Region[which(datos$Latitude==lat[i] & datos$Longitude==lon[j])] #lat lon
  }
}
save(cp,file="regionesMArtigas_0.1.Rdata")

#############################################

#Useful information
calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
regions <- load.Rdata2("regionesMArtigas_0.1.Rdata")

#Load CPs information
load("WardCluster_Dendrogram_D4_50km.Rdata")
k=8
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1951:2020))]
cp <- loadGridData(dataset = "eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),years=1951:2020,var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))

load("distance2_D4.Rdata")
calendario <- seq(as.Date("1950-01-01"),as.Date("2022-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

distance3 <- distance2[,which(format(calendario,"%Y")%in%c(1951:2020))]


#________________SUBSEASONAL (MONTHLY)________________#
season <- c("Jan"=1,"Feb"=2,"Mar"=3,"Apr"=4,"May"=5,"Jun"=6,
            "Jul"=7,"Aug"=8,"Sep"=9,"Oct"=10,"Nov"=11,"Dec"=12)

for(r in 1:3){
  reg <- regions
  reg$Data[which(regions$Data!=r)] <- 0
  reg$Data[which(regions$Data==r)] <- 1
  
  #____________________tasmin_______________________________#
  #Load (monthly) surface data
  nc <- loadGridData(dataset="aemet-5km_mon/tasmin_mon_aemet-5km_1951-2020.nc",var="tasmin")
  nc <- interpGrid(nc,new.coordinates = getGrid(regions))
  
  for(y in 1:dim(nc$Data)[1]){
    nc$Data[y,,] <- nc$Data[y,,]*reg$Data
  }
  
  data <- list(); value <- matrix(NA,k,12) ; sign <- matrix(NA,k,12)
  for(i in 1:12){
    mon_intann <- subsetGrid(nc,season=i)
    index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
    
    #Generating time-series for each CP
    for(l in 1:length(unique(vector_cluster))){
      for(le in 1:length(cp$Data)){
        cp$Data[le] <- distance3[l,le]
      }
      
      index.cp <- aggregateGrid(redim(cp),aggr.m=list(FUN="mean"))

      #Selecting the month/season of interest
      index.cp <- subsetGrid(index.cp,season=i)
      
      #Removing trend in CP frequency (if any)
      trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
      index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
      
      #Removing trend in surface variable (if any)
      trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
      index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
      
      value[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
      sign[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
    }
    
  }
  
  data <- data.frame(Index=as.numeric(value)*-1,
                     Season=rep(names(season),each=8),
                     CP=rep(c(1:8),12),
                     Sign=as.numeric(sign))
  
  datitos.cor <- data
  datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))
  
  g <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                          aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
    ggtitle("TN")+
    geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
    geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)
  
  #____________________tasmax_______________________________#
  nc <- loadGridData(dataset="aemet-5km_mon/tasmax_mon_aemet-5km_1951-2020.nc",var="tasmax")
  nc <- interpGrid(nc,new.coordinates = getGrid(regions))
  
  for(y in 1:dim(nc$Data)[1]){
    nc$Data[y,,] <- nc$Data[y,,]*reg$Data
  }
  
  data <- list(); value <- matrix(NA,k,12) ; sign <- matrix(NA,k,12)
  for(i in 1:12){
    mon_intann <- subsetGrid(nc,season=i)
    index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
    
    #Generating time-series for each CP
    for(l in 1:length(unique(vector_cluster))){
      for(le in 1:length(cp$Data)){
        cp$Data[le] <- distance3[l,le]
      }
      
      index.cp <- aggregateGrid(redim(cp),aggr.m=list(FUN="mean"))
      #Selecting the month/season of interest
      index.cp <- subsetGrid(index.cp,season=i)
      
      #Removing trend in CP frequency (if any)
      trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
      index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
      
      #Removing trend in surface variable (if any)
      trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
      index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
      
      value[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
      sign[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
    }
    
  }
  
  data <- data.frame(Index=as.numeric(value)*-1,
                     Season=rep(names(season),each=8),
                     CP=rep(c(1:8),12),
                     Sign=as.numeric(sign))
  
  datitos.cor <- data
  datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))
  
  h <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                          aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
    ggtitle("TX")+
    geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
    geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)
  
  #____________________precip_______________________________#
  nc <- loadGridData(dataset="aemet-5km_mon/prlr_mon_aemet-5km_1951-2020.nc",var="prlr")
  nc <- interpGrid(nc,new.coordinates = getGrid(regions))
  
  for(y in 1:dim(nc$Data)[1]){
    nc$Data[y,,] <- nc$Data[y,,]*reg$Data
  }
  
  data <- list(); value <- matrix(NA,k,12) ; sign <- matrix(NA,k,12)
  for(i in 1:12){
    mon_intann <- subsetGrid(nc,season=i)
    index2 <- aggregateGrid(mon_intann,aggr.spatial = list(FUN="mean",na.rm=T)) #acá va la de tal region
    
    #Generating time-series for each CP
    for(l in 1:length(unique(vector_cluster))){
      for(le in 1:length(cp$Data)){
        cp$Data[le] <- distance3[l,le]
      }
      
      index.cp <- aggregateGrid(redim(cp),aggr.m=list(FUN="mean"))  
      #Selecting the month/season of interest
      index.cp <- subsetGrid(index.cp,season=i)
      
      #Removing trend in CP frequency (if any)
      trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
      index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
      
      #Removing trend in surface variable (if any)
      trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
      index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
      
      value[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
      sign[l,i] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
    }
    
  }
  
  data <- data.frame(Index=as.numeric(value)*-1,
                     Season=rep(names(season),each=8),
                     CP=rep(c(1:8),12),
                     Sign=as.numeric(sign))
  datitos.cor <- data
  datitos.cor$Season <- factor(datitos.cor$Season,levels=names(season))
  
  p <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Index)) + geom_tile() + theme_bw() + 
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                          aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
    ggtitle("PR")+
    geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
    geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Index,digits=2))) + scale_y_continuous(n.breaks=7)
  
  #___
  png(filename = paste(OUTPUTS,"Cor_aemet-5km_","CPs_D4_RegionMA_",r,"_Subseasonal_Distances.png"),res=300,width=4000,height=4000)
  print(grid.arrange(p,g,h,ncol=1))
  dev.off()
  
  ##################################################################################
  #season <- list("Winter"=c(12,1,2),"Spring"=c(3,4,5),"Summer"=c(6,7,8),"Autumn"=c(9,10,11),"Annual"=c(1:12))
  
}

png(filename = paste(OUTPUTS,"Regiones_3_MA.png"),res=300,width=2000,height=2000)
spatialPlot(regions,backdrop.theme = "countries",color.theme = "Paired")
dev.off()
