#Differentiated model ensembles based on historical performance
library(ggplot2)

rm(list=ls()) ; gc()

dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CMIP6/"
setwd(dir)

scenario <- c("ssp585","ssp245","ssp126")
region <- c("EuroMediterranean","Catalonia")


OUTPUTS <- "/esarchive/scratch/molmo/"

f <- function(x) {
  r <- quantile(x, probs = c(0.05, 0.25, 0.5, 0.75, 0.95))
  names(r) <- c("ymin", "lower", "middle", "upper", "ymax")
  r
}

load("/esarchive/scratch/molmo/output/D4/orden_DISO.Rdata")
best_gcms <- c("EC-Earth3","EC-Earth3-CC","EC-Earth3-Veg","EC-Earth3-Veg-LR","MIROC6",
               "GFDL-CM4","GDFL-ESM4","IPSL-CM6A-LR","ACCESS-CM2","TaiESM1","BCC-CSM2-MR",
               "CMCC-CM2-SR5","IITM-ESM","MPI-ESM1-2-HR","NorESM2-LM")
orden2 <- NA
for(g in 1:length(orden)){
  orden2[g] <- as.character(substr(as.character(orden[g]),start = 3,stop=nchar(as.character(orden[g]))))
  orden2[g] <- trimws(orden2[g])
}

best_gcms <- orden2[1:10]
#best_gcms <- c("EC-Earth3-CC","EC-Earth3-Veg","GFDL-CM4","GDFL-ESM4","IPSL-CM6A-LR","MIROC6","MPI-ESM1-2-HR","TaiESM1")

###START###
for(sc in 1:length(scenario)){
  message(scenario[sc])
  for(r in 1:2){
    message(region[r])
    Region <- region[r]
   
    #---Seasonal---#
    load(paste("series.indices.mon.CMIP6.",scenario[sc],".Rdata",sep=""))
    indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
                 "rx5day","rx1day")
    
    test <- list()
    for(i in 1:length(indices)){
      test[[i]] <- list()
      for(m in 1:length(series.indices[[i]])){
        test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
      }
    }
    
    test2 <- list()
    for(i in 1:length(indices)){
      test2[[i]] <- do.call("rbind",test[[i]])
    }
    
    test3 <- do.call("rbind",test2)
    test3$Index <- toupper(test3$Index)
    
    Model <- unique(test3$Model)
    Model <- Model[order(Model)]
    
    test3$Index <- factor(test3$Index,levels=toupper(c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                                       "rx1day","rx5day")))
    
    test3$Season <- factor(test3$Season,levels=c("Summer","Autumn","Winter","Spring"))
    test4 <- test3
    
    test3 <- subset(test4,Region==region[r])

    all <- unique(test3$Model)
    rest_gcms <- all[-which(all%in%best_gcms)]
    
    test3$Performance <- NA
    test3$Performance[which(test3$Model%in%best_gcms)] <- "Good"
    test3$Performance[which(test3$Model%in%rest_gcms)] <- "Rest"
    
    test3_fut <- subset(test3,Year%in%c(2070:2100))
    prueba <- aggregate(Serie~Index+Season+Performance+Region+Model,test3_fut,FUN="median",na.rm=T)

    png(filename = paste(OUTPUTS,"Boxplot_Changes_Seasonal",scenario[sc],"_",Region,"_",length(best_gcms),".png",sep=""),res=300,width=3000,height=2000)
    print(
      ggplot(data=prueba,aes(x=Performance,y=Serie,fill=Performance)) + theme_bw() + stat_summary(alpha=0.95,fun.data=f, geom="boxplot",position="dodge")+ geom_vline(xintercept = 0,color="black")+# coord_cartesian(expand=F)+
        ggtitle("Seasonal")+ facet_grid(Season~Index,scales = "free_x")+coord_flip() + theme(legend.position = "bottom")#+
    )
    dev.off() 
    
    #---Annual---#
    load(paste("series.indices.yr.CMIP6.",scenario[sc],".Rdata",sep=""))
    indices <- c("wsdi","csdi","tr",
                 "r20mm","prcptot","cdd","cwd")
    
    test <- list()
    for(i in 1:length(indices)){
      test[[i]] <- list()
      for(m in 1:length(series.indices[[i]])){
        test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
      }
    }
    
    test2 <- list()
    for(i in 1:length(indices)){
      test2[[i]] <- do.call("rbind",test[[i]])
    }
    
    test3 <- do.call("rbind",test2)
    Model <- unique(test3$Model)

    test3$Index <- toupper(test3$Index)
    test3$Index <- factor(test3$Index,levels=toupper(c("cdd","cwd","prcptot","r20mm","csdi","wsdi","tr")))
    
    test4 <- test3
    
    test3 <- subset(test4,Region==region[r])
    
    all <- unique(test3$Model)
    rest_gcms <- all[-which(all%in%best_gcms)]
    
    test3$Performance <- NA
    test3$Performance[which(test3$Model%in%best_gcms)] <- "Good"
    test3$Performance[which(test3$Model%in%rest_gcms)] <- "Rest"
    
    test3_fut <- subset(test3,Year%in%c(2070:2100))
    prueba <- aggregate(Serie~Index+Season+Performance+Region+Model,test3_fut,FUN="median",na.rm=T)
    
    png(filename = paste(OUTPUTS,"Boxplot_Changes_Annual",scenario[sc],"_",Region,"_",length(best_gcms),".png",sep=""),res=300,width=3000,height=2000)
    print(
      ggplot(data=prueba,aes(x=Performance,y=Serie,fill=Performance)) + theme_bw() + stat_summary(alpha=0.95,fun.data=f, geom="boxplot",position="dodge")+ geom_vline(xintercept = 0,color="black")+# coord_cartesian(expand=F)+
        ggtitle("Seasonal")+ facet_grid(Season~Index,scales = "free_x")+coord_flip() + theme(legend.position = "bottom")#+
    )
    dev.off() 
    
  }
}
